function [dirichleteigs,neumanneigs,p1,e1,t1,Ss,ND1] = accwave1(nrefine,Jmax,nchannel,lambdamin,lambdamax,lambda0);
format long;

%nrefine=2;
%lambdamin=-10;
%lambdamax=100;

x1 = 0;
y1 = -2;
x2 = sqrt(2);
y2 = -x2;
x3 = 1/x2;
y3 = 1/y2;
x4 = 0;
y4 = -1;

g = [ 4.0   2.0    4.0    2.0  ;
      x1    x2     x4     x4   ;
      x2    x3     x3     x1   ;
      y1    y2     y4     y4   ;
      y2    y3     y3     y1   ;
      1.0   1.0    0.0    1.0  ;
      0.0   0.0    1.0    0.0  ;
      0     0      0      0    ;
      0     0      0      0    ;
      2     0      1      0    ;
      2     0      1      0    ;
      0     0      0      0   ];

[p,e,t]=initmesh(g);

p1 = p; e1 = e; t1 = t;
for j = 1:nrefine-1
[p1,e1,t1]=refinemesh(g,p1,e1,t1);
end;
figure(1), clf, pdemesh(p1,e1,t1), axis equal, title('Refined mesh');
hold;

% Mixed boundary condition on boundary 1 (column 4 of geometry matrix):
for j = 1:Jmax
% This specifies that the normal derivative of u on the boundary is the
% 2j-1 th basis function.
    bm=bmixed1(4,[1 2],3,j);
    u(:,2*j-1)=assempde(bm,p1,e1,t1,1.0,-lambda0,0.0);
end;
usaved = u(:,1);
for j = 1:Jmax
% This specifies that the normal derivative of u on the boundary is the 
% 2j th basis function.
    bm=bmixed2(2,[1 4],3,j);
    u(:,2*j)=assempde(bm,p1,e1,t1,1.0,-lambda0,0.0);
end;

bd = bdir_0(4);
% Get Dirichlet eigenvalues and eigenfunctions:
[vloc,dirichleteigs]=pdeeig(bd,p1,e1,t1,1.0,0.,1.0,[lambdamin lambdamax]);

bidon = bneum_0(4);

% Get Neumann eigenvalues and eigenfunctions:
[vloc,neumanneigs]=pdeeig(bidon,p1,e1,t1,1.0,0.,1.0,[lambdamin lambdamax]);

% Normalize Neumann eigenfunctions using stiffness matrix:
[K1,M1,F1]=assema(p1,t1,0.,1.,0.);
for m=1:length(neumanneigs)
% Normalize m-th eigenfunction to have L^2-norm equal to 1:
   v(:,m)=(vloc(:,m)'*M1*vloc(:,m))^(-1/2)*vloc(:,m);
end;

% Get indices of meshpoints on chopped off ends:
bindices1=find( (abs(p1(1,:))< 0.01))
bindices2=find( abs(p1(1,:)+p1(2,:))<0.01)
%plot(p1(1,bindices2),p1(2,bindices2),'*')

% Get coordinates of the points on the boundary portion 1 and arrange in
% anticlockwise order.
ybound1 = p1(2,bindices1);
[yb1,iyb1] = sort(-ybound1);
bindex1 = bindices1(iyb1);
yb1 = p1(2,bindex1);

% Get coordinates of the points on the boundary portion 2 and arrange in 
% anticlockwise order:
ybound2 = p1(2,bindices2);
[yb2,iyb2] = sort(ybound2);
bindex2 = bindices2(iyb2);
yb2 = p1(2,bindex2);
sb2 = yb2*sqrt(2)+2;

% assembling basis functions Psi1 on boundary 1:
Psi1=zeros(length(bindex1),Jmax);
for j = 1:Jmax
% For some weird reason it seems to be necessary to have the `wrong'
% sign on this component in order to get the right sign in the
% Neumann-Dirichlet map ND1. The PDE toolbox has some weird
% convention on the direction of the normal derivative.

Psi1(:,j) = sqrt(2)*sin((j-1/2)*pi*(yb1'+1));
end;

% assembling basis functions Psi2 on boundary 2:
Psi2=zeros(length(bindex2),Jmax);
for j=1:Jmax
    Psi2(:,j) =  sqrt(2)*sin((j-1/2)*pi*(1-sb2'));
end;

%%%
%%% integrating for each eigenfunction
%%%

Ss = zeros(length(neumanneigs),2*Jmax);

vix1 = zeros(1,length(bindex1)+1);
vix2 = zeros(1,length(bindex2)+1);

for m=1:size(u,2)
    ub1 = u(bindex1,m);
    ub2 = u(bindex2,m);
    for n = 1:Jmax
        ND1(m,2*n-1) = trapz(yb1',ub1.*Psi1(:,n));
        ND1(m,2*n) = trapz(sb2',ub2.*Psi2(:,n));
    end;
end;


for m=1:size(v,2)
% Traces of eigenfunctions on the first part of the boundary:
   vb1=v(bindex1,m);
% Traces of eigenfunctions on the second part of the boundary:
   vb2=v(bindex2,m);
%   size(vb1)
%   size(Psi1)
%   size(vb1.*Psi1(:,3))
%   size(yb1)
   for n = 1:Jmax
       Ss(m,2*n-1) = trapz(yb1',vb1.*Psi1(:,n));
       Ss(m,2*n) = trapz(sb2',vb2.*Psi2(:,n));
   end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Functions stolen from Michael:

function [b_matr]=bneum_0(s)

b_matr=repmat([1;0;1;1;48;48;48;49;48],1,s); %%% homogeneous Neumann boundary condition on ALL s pieces
% Change to Dirichlet on piece 3:
b_matr(1:10,3)=repmat([1;1;1;1;1;1;'0';'0';'1';'1'],1,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55
%%%

function [b_matr]=bdir_0(s)

b_matr=repmat([1;0;1;1;48;48;48;49;48],1,s); %%% homogeneous Neumann boundary condition on ALL pieces
%% Change to Dirichlet on  boundary pieces 2, 3 and 4:
b_matr(1:10,2:4)=repmat([1;1;1;1;1;1;'0';'0';'1';'1'],1,3);

